// Copyright 2016 Concrete Software. All rights reserved.

uniform sampler2D texture;
// Provides noise while rematerializing
uniform sampler2D pba_NoiseTexture;
// Amount to offset texture by is determined by time
uniform mediump float pba_Time;
// Direction of the main light in eye coordinates. Must be normalized.
uniform mediump vec3 pba_KeyLightPosition;
// Shininess amount. 128 is very shiny. 1 is not so shiny.
uniform mediump float pba_KeyShininess;
// values for "Rim" light on side to bring out shape
uniform mediump vec3 pba_RimLightPosition;
//uniform lowp float pba_RimShininess;
// color (and intensity) for the diffuse light
uniform mediump vec3 pba_diffuseColor;
// color (and intensity) for the ambient light
uniform mediump vec3 pba_ambientColor;
// color (and intensity) for the specular component of the light
uniform mediump vec3 pba_specularColor;

varying mediump vec3 var_Normal;
varying mediump vec3 var_TransformedVertex;
varying mediump vec2 var_TexCoord;
varying mediump vec2 var_ScreenCoord;

void main()
{
    // we'll eventually need to blend the stuff below in...
    mediump float noiseValue = texture2D(pba_NoiseTexture, var_ScreenCoord + pba_Time * vec2(10.3, 14.7)).r;
    
    // We want an outline around the edge of the pin. We can get this by looking for spots where the normal is almost perpendicular to the camera
	mediump vec3 eyePosition = normalize(var_TransformedVertex);
	mediump vec3 normalizedNormal = normalize(var_Normal);
    
    noiseValue *= 1.0-pow(1.-abs(dot(eyePosition, normalizedNormal)), 2.);
    
    // color where noiseValue == 0. Will be white elsewhere
    mediump vec3 colorForNoise = vec3(0.15686275, 0.70196078, 0.5372549);
    mediump vec3 toWhite = 1. - colorForNoise;
    
    mediump vec3 noiseColor = colorForNoise + toWhite * noiseValue;
    
    // Now get the normal color of the pin to blend in
	mediump float diffuseIntensity = max(dot(normalizedNormal, pba_KeyLightPosition), 0.0);
    diffuseIntensity += max(dot(normalizedNormal, pba_RimLightPosition), 0.0);

	mediump vec3 halfLight = normalize(pba_KeyLightPosition - eyePosition);
	mediump float normalDotHalf = max(dot(normalizedNormal, halfLight), 0.);
	mediump float specularComponent = clamp(pow(normalDotHalf, pba_KeyShininess), 0., 1.);

	mediump vec3 normalColor = vec3(pba_ambientColor + pba_diffuseColor * diffuseIntensity) * texture2D(texture, var_TexCoord).rgb + vec3(specularComponent * pba_specularColor);
    
    // pin fades from transparent to opaque noise over first half of transition
    mediump float alpha = clamp(pba_Time * 2., 0., 1.);
    // pin fades from noise to normal color over second half of transition
    mediump float blend = clamp(pba_Time * 2. - 1., 0., 1.);
    
    gl_FragColor = vec4(mix(noiseColor, normalColor, blend) * alpha, alpha);
    
}
