// Copyright 2013 Concrete Software. All rights reserved.

#ifdef BUMPMAP
uniform mediump mat3 cs_NormalMatrix;
uniform sampler2D pba_BumpTexture;

varying mediump vec3 var_Bitangent;
varying mediump vec3 var_Tangent;

// space between samples in bump texture (1/512)
#define DELTA 0.001953125
// Adjusts the height of bumps. This is equal to 1/(amount of change needed in the sample to be represented as a 45 degree angle.) The old default value was 2.
uniform mediump float pba_BumpHeight;
#endif
#ifdef SPECMAP
uniform sampler2D pba_SpecTexture;
#endif

uniform sampler2D texture;
// Direction of the main light in eye coordinates. Must be normalized.
uniform mediump vec3 pba_KeyLightPosition;
// Shininess amount. 128 is very shiny. 1 is not so shiny.
uniform mediump float pba_KeyShininess;
// values for "Rim" light on side to bring out shape
uniform mediump vec3 pba_RimLightPosition;
//uniform lowp float pba_RimShininess;
// color (and intensity) for the diffuse light
uniform mediump vec3 pba_diffuseColor;
// color (and intensity) for the ambient light
uniform mediump vec3 pba_ambientColor;
// color (and intensity) for the specular component of the light
uniform mediump vec3 pba_specularColor;

varying mediump vec3 var_Normal;
varying mediump vec3 var_TransformedVertex;
varying mediump vec2 var_TexCoord;

void main()
{
#ifdef BUMPMAP
    // bump map is single channel only, so taking any channel (not just red) would work.
    mediump float height = texture2D(pba_BumpTexture, var_TexCoord).r;
    mediump float heightDX = (height-texture2D(pba_BumpTexture, vec2(var_TexCoord.x+DELTA, var_TexCoord.y)).r)*pba_BumpHeight;
    mediump float heightDY = (height-texture2D(pba_BumpTexture, vec2(var_TexCoord.x, var_TexCoord.y+DELTA)).r)*pba_BumpHeight;
    // 0, 0, 1 is just a normal smooth surface.
    highp vec3 normal = normalize(vec3(heightDX,heightDY,max(1.-heightDX*heightDX-heightDY*heightDY, 0.)));
    
    // transform normal back to object space
    normal = mat3(var_Tangent, var_Bitangent, var_Normal) * normal;
    
	mediump vec3 normalizedNormal = normalize(cs_NormalMatrix * normal);
#else
	mediump vec3 normalizedNormal = normalize(var_Normal);
#endif
	
	mediump float diffuseIntensity = max(dot(normalizedNormal, pba_KeyLightPosition), 0.0);
    diffuseIntensity += max(dot(normalizedNormal, pba_RimLightPosition), 0.0);
	
	mediump vec3 eyePosition = normalize(var_TransformedVertex);
	mediump vec3 halfLight = normalize(pba_KeyLightPosition - eyePosition);
	mediump float normalDotHalf = max(dot(normalizedNormal, halfLight), 0.);
	mediump float specularComponent = clamp(pow(normalDotHalf, pba_KeyShininess), 0., 1.);
    
	gl_FragColor = vec4(pba_ambientColor + pba_diffuseColor * diffuseIntensity, 1.0) * texture2D(texture, var_TexCoord) + vec4(specularComponent * pba_specularColor, 1.0)
    #ifdef SPECMAP
    * texture2D(pba_SpecTexture, var_TexCoord)
    #endif
    ;
}
