// reflective_floor.fs
// Copyright 2016 Concrete Software, Inc.
//
//

// Normal floor texture
uniform sampler2D texture;
// Reflection map
uniform sampler2D pba_ReflectionTexture;
// Floor normal map
uniform sampler2D pba_NormalMap;
// Amount of distortion in the reflection for the bumps.
uniform mediump float pba_BumpHeight;
// How strong the floor's texture should be in the mix. 1 means that there
// are no reflections (and this shader is a waste of time.) 0 means that the original texture has no effect and only the reflections are visible.
uniform mediump float pba_FloorOpacity;
// Standard texture coordinates
varying mediump vec2 var_TexCoord;
// x,y,z screen coordinates. (x,y)/z will go from 0..1 and can be used to
// look into the reflection texture.
varying mediump vec3 var_ScreenCoord;

void main()
{
    // color of the floor texture here
    mediump vec4 textureColor = texture2D(texture, var_TexCoord);
    // Get the normal map value. Only care about the red and green channels; b holds z which is "up" relative to the floor.
    mediump vec2 bump = texture2D(pba_NormalMap, var_TexCoord).rg;
    // turn normal map value 0~1 into a normal -1~1
    bump = bump * 2. - 1.;
    // Get the screen coordinate, perturbed by the normal. Divide out z here so that things that are far away are not perturbed as much as things close by.
    mediump vec2 screenCoord = (bump * pba_BumpHeight + var_ScreenCoord.xy) / var_ScreenCoord.z;
    mediump vec4 reflectionColor = texture2D(pba_ReflectionTexture, screenCoord);
    gl_FragColor = mix(reflectionColor, textureColor, pba_FloorOpacity);
}
