// reflective_floor.vs
// Copyright 2016 Concrete Software, Inc.
//

uniform mat4 cs_ModelViewProjectionMatrix;

attribute vec4 cs_Vertex;
attribute vec2 cs_MultiTexCoord0;

varying mediump vec2 var_TexCoord;
varying mediump vec3 var_ScreenCoord;

void main()
{
    vec4 screenCoord = cs_ModelViewProjectionMatrix * cs_Vertex;
    gl_Position = screenCoord;
    var_TexCoord = cs_MultiTexCoord0;
    // This is passed to fragment shader. The position on the screen (used above) is
    // (x / w, y / w)
    // However, we don't actually want a position on a screen, which has coordinates
    // from -1~1. We want a position in our reflection texture, which is 0~1.
    // So, we convert it here:
    // ((x / w, y / w) + 1) / 2 = (x + w, y + w) / 2w
    // Note: we don't actually do the division here because we need the w coordinate
    // in the fragment shader so that we can move the screen coordinate by more when
    // it is closer and less when it is further away
    var_ScreenCoord = vec3((screenCoord.xy + screenCoord.w), screenCoord.w * 2.);
}
