// Fragment shader for an old-timey look on the hall of fame lane
// Copyright 2017 Concrete Software. All rights reserved.

uniform sampler2D texture;
varying mediump vec2 var_TexCoord;
varying mediump vec2 var_ScreenPos;

uniform mediump float pba_Time;
uniform lowp float pba_TransitionPct;

lowp float greyscale(lowp vec4 color) {
    return dot(color.rgb, vec3(0.3, 0.59, 0.11));
}

//Based on https://www.shadertoy.com/view/ldXGW4

// Noise generation functions borrowed from:
// https://github.com/ashima/webgl-noise/blob/master/src/noise2D.glsl

mediump vec2 mod289(mediump vec2 x) {
    return x - floor(x * (1.0 / 289.0)) * 289.0;
}

mediump vec3 mod289(mediump vec3 x) {
    return x - floor(x * (1.0 / 289.0)) * 289.0;
}

mediump vec3 permute(mediump vec3 x) {
    return mod289(((x*34.0)+1.0)*x);
}

mediump float snoise(mediump vec2 v)
{
    const mediump vec4 C = vec4(0.211324865405187,  // (3.0-sqrt(3.0))/6.0
                        0.366025403784439,  // 0.5*(sqrt(3.0)-1.0)
                        -0.577350269189626,  // -1.0 + 2.0 * C.x
                        0.024390243902439); // 1.0 / 41.0
    // First corner
    mediump vec2 i  = floor(v + dot(v, C.yy) );
    mediump vec2 x0 = v -   i + dot(i, C.xx);
    
    // Other corners
    mediump vec2 i1;
    //i1.x = step( x0.y, x0.x ); // x0.x > x0.y ? 1.0 : 0.0
    //i1.y = 1.0 - i1.x;
    i1 = (x0.x > x0.y) ? vec2(1.0, 0.0) : vec2(0.0, 1.0);
    // x0 = x0 - 0.0 + 0.0 * C.xx ;
    // x1 = x0 - i1 + 1.0 * C.xx ;
    // x2 = x0 - 1.0 + 2.0 * C.xx ;
    mediump vec4 x12 = x0.xyxy + C.xxzz;
    x12.xy -= i1;
    
    // Permutations
    i = mod289(i); // Avoid truncation effects in permutation
    mediump vec3 p = permute( permute( i.y + vec3(0.0, i1.y, 1.0 ))
                     + i.x + vec3(0.0, i1.x, 1.0 ));
    
    mediump vec3 m = max(0.5 - vec3(dot(x0,x0), dot(x12.xy,x12.xy), dot(x12.zw,x12.zw)), 0.0);
    m = m*m ;
    m = m*m ;
    
    // Gradients: 41 points uniformly over a line, mapped onto a diamond.
    // The ring size 17*17 = 289 is close to a multiple of 41 (41*7 = 287)
    
    mediump vec3 x = 2.0 * fract(p * C.www) - 1.0;
    mediump vec3 h = abs(x) - 0.5;
    mediump vec3 ox = floor(x + 0.5);
    mediump vec3 a0 = x - ox;
    
    // Normalise gradients implicitly by scaling m
    // Approximation of: m *= inversesqrt( a0*a0 + h*h );
    m *= 1.79284291400159 - 0.85373472095314 * ( a0*a0 + h*h );
    
    // Compute final noise value at P
    mediump vec3 g;
    g.x  = a0.x  * x0.x  + h.x  * x0.y;
    g.yz = a0.yz * x12.xz + h.yz * x12.yw;
    return 130.0 * dot(m, g);
}

#define SQRT_2 1.4142135624
lowp float crosswipe(lowp vec2 uv, lowp float pct) {
    const lowp float radius = .5;
    const lowp float smoothness = .4;

    pct /= 1.0 - smoothness;
    return smoothstep(radius,radius-smoothness,pct - length(uv) / SQRT_2 + radius - smoothness);
}

lowp float calcTransition(lowp vec2 uv) {
    return crosswipe(uv, pba_TransitionPct);
}

void main()
{
    mediump vec2 uv = var_TexCoord;

    lowp float transition = calcTransition(uv);
    mediump float fuzzOffset = snoise(vec2(pba_Time*15.0,uv.y*80.0))*0.0015 * transition;
    mediump float largeFuzzOffset = snoise(vec2(pba_Time*1.0,uv.y*25.0))*0.001 * transition;
    
    mediump float xOffset = (fuzzOffset + largeFuzzOffset);
    uv.x += xOffset;
    
    lowp vec4 color = texture2D(texture, uv);
    
    lowp float grey = greyscale(color);
    mediump float scanline = sin(uv.y*800.0)*0.1;
    grey -= scanline;
    
    color = mix(color, vec4(grey), transition);
    
    gl_FragColor = vec4(color.rgb,1.0);
}

